/** @file josephus.c
 * @brief Solves the Josephus problem.
 *
 * For this program, consider that it takes two integers from the command line, 
 * n and m. The integers from 1 to n are queued, and then you must dequeue an  
 * element every m places. 
 *
 * ./josephus 9 5
 * 5 1 7 4 3 6 9 2 8 
 *
 * @author Pavlos Antoniou
 * @bug No known bugs
 */
 
#include <stdio.h>
#include <stdlib.h>


#define EMPTY -1


typedef struct node {
  int data;
  struct node* next;
}Node;


typedef struct queue {
  Node* head;
  Node* tail;
}Queue;


/** @brief Initializes a queue by properly setting members head & tail.
 *
 * @param queue valid struct queue
 * @post the struct queue points to, will be set up to represent an empty queue
 * @return void
*/
int initQueue(Queue** queue);


/** 
 * @brief Adds val to the tail of the queue.
 * @param qPtr points to a valid struct queue
 * @param val is the value to enqueue into the queue pointed to by qPtr
 * @return 1 If the operation is successful
 *         0 If not successfull, & no change will be made to the queue
 */
int qEnqueue(Queue* qPtr, int val);


/** @brief Dequeues one element from the head of the queue.
 *
 * @param qPtr points to a valid Queue.
 * @post  If the queue pointed to by qPtr is non-empty, then the value at the
 *            head of the queue is deleted from the queue and returned.
 *         Otherwise, -1 is returned to signify that the queue was already empty
 *            when the dequeue was attempted.
 *  @return value at the head of the queue If the queue is non-empty
 *          -1 if the queue was already empty
 */
int qDequeue(Queue* qPtr);


/** @brief Checks to see if the queue is empty.
 *
 * @param qPtr points to a valid Queue.
 * @return true if the queue pointed by qPtr is empty
 */
int qEmpty(Queue* qPtr);


/** @brief Returns the data of qFront from a struct qPtr.
 *
 * @param qPtr points to a valid Queue.
 * @return value at head  if the queue pointed by qPtr is non-empty
 *         -1              if the queue is empty
 */
int qHead(Queue* qPtr);


//===========================================================================//

int initQueue(Queue** queue) {
  *queue = (Queue *)malloc(sizeof(Queue));
  if ((*queue) == NULL) 
    return EXIT_FAILURE;
  // Just set both pointers to NULL!
  (*queue)->head = (*queue)->tail = NULL;
  return EXIT_SUCCESS;
}


int qEnqueue(Queue* qPtr, int val) {

  struct node* temp;
  
  // Allocate space for a new node to add into the queue.
  temp = (struct node*)malloc(sizeof(struct node));
  
  // This case checks to make sure our space got allocated.
  if (temp != NULL) {
    // Set up our node to enqueue into the tail of the queue.
    temp->data = val;
    temp->next = NULL;
    
    // If the queue is NOT empty, we must set the old "last" node to point
    // to this newly created node.
    if (qPtr->tail != NULL)
      qPtr->tail->next = temp;
    
    // Now, we must reset the tail of the queue to our newly created node.
    qPtr->tail = temp;
    
    // If the queue was previously empty we must ALSO set the head of the
    // queue.
    if (qPtr->head == NULL)
      qPtr->head = temp;
    
    // Signifies a successful operation.
    return 1;
  }
  // No change to the queue was made because we couldn't find space for our
  // new enqueue.
  else
    return 0;   
}


int qDequeue(Queue* qPtr) {
    
  struct node* tmp;
  int retval;
  
  // Check the empty case.
  if (qPtr->head == NULL)
    return EMPTY;
    
  // Store the head value to return.
  retval = qPtr->head->data;
  
  // Set up a temporary pointer to use to free the memory for this node.
  tmp = qPtr->head;
  
  // Make head point to the next node in the queue.
  qPtr->head = qPtr->head->next;
  
  // If deleting this node makes the queue empty, we have to change the tail
  // pointer also!
  if (qPtr->head == NULL)
    qPtr->tail = NULL;
  
  // Free our memory.
  free(tmp);
  
  // Return the value that just got dequeued.
  return retval;
}


int qHead(Queue* qPtr) {
  if (qPtr->head != NULL)
    return qPtr->head->data;
  else
    return EMPTY;
}


int qEmpty(Queue* qPtr) {
  return qPtr->head == NULL;
}


int main(int argc, char **argv) {
  int i;

  if (argc != 3) {
    fprintf(stderr,
	  " usage:\n"
	  " ./josephus n m \n"
	  " n : the first n integers\n"
	  " m : the step for eliminating an integer\n"
	  "\n");
    exit(1);
  }
  int n = atoi(argv[1]);
  int m = atoi(argv[2]);

  // initialize the queue
  Queue *josephusQueue;// = (Queue *)malloc(sizeof(Queue));;
  initQueue(&josephusQueue);

  for (i = 1; i <= n; i++)
    qEnqueue(josephusQueue,i);

  while (!qEmpty(josephusQueue)) {
    for (i = 0; i < m - 1; i++)
      qEnqueue(josephusQueue, qDequeue(josephusQueue));
    printf("%d ",qDequeue(josephusQueue));
  } 
  printf("\n");

  free(josephusQueue);

  return 0;
}

