/** @file parentheses.c
 * @brief Check for balanced parenthesis
 *
 * Reads in a text stream from standard input and uses a stack to determine 
 * whether its parentheses are properly balanced. For example, your program 
 * should print true for [()]{}{[()()]()} and false for [(]). 
 *
 * @author Pavlos Antoniou
 * @bug No known bugs
 */
 
#include <stdio.h>
#include <stdlib.h>

/*
 * The stack structure
 */
typedef struct stack {
  int value;
  struct stack *prev;
} stack ;


/** @brief Push an element to the stack
 *
 * @param stack** the stack
 * @param int the element
 * @return void
 */
void push(stack **, int);


/** @brief Pop value from the stack
 *
 * @param stack** the stack
 * @return int the element popped from stack
 */
int pop(stack **);


/** @brief Clear the stack
 *
 * Deletes the elements of the stack. Free the memory.
 *
 * @param stack** the stack
 * @return void
 */
void clear(stack **);


/** @brief Returns the number of elements in the stack
 *
 * @param stack* the stack
 * @return int number of elements
 */
int length(stack *p);


/** @brief Displays the stack
 *
 * @param stack* the stack
 * @return void
 */
void display(stack *);
           

/*************************************************************************/

void push(stack **p, int val) {
  stack *element = (stack *)malloc(sizeof(stack));
  if(!element) 
    exit(EXIT_FAILURE);   
  
  element->value = val;
  element->prev = *p;
  *p = element;       
}


int pop(stack **p) {
  int val;
  stack *tmp;
  if(!*p) return -1;     /* Retourne -1 si la stack est vide. */
  tmp = (*p)->prev;
  val = (*p)->value;
  free(*p);
  *p = tmp;       /* Le pointeur pointe sur le dernier élément. */
  return val;     /* Retourne la value soutirée de la stack. */
}



void clear(stack **p) {
  stack *tmp;
  while(*p) {
    tmp = (*p)->prev;
    free(*p);
    *p = tmp;
  }
}


int length(stack *p) {
  int n=0;
  while(p) {
    n++;
    p = p->prev;
  }
  return n;
}



void display(stack *p) {
  while(p) {
    printf("%d\n",p->value);
    p = p->prev;
  }
}

/*************************************************************************/


int main (int argc, char **argv) {

  int i;
  char c;

  char L_PAREN    = '(';
  char R_PAREN    = ')';
  char L_BRACE    = '{';
  char R_BRACE    = '}';
  char L_BRACKET  = '[';
  char R_BRACKET  = ']';

  stack *parStack;
  parStack = NULL;

  printf("Enter a string: ");

  while ((c = getchar()) != '\n') {
    if (c == L_PAREN)
      push(&parStack, L_PAREN);
    else 
      if (c == L_BRACE)
	push(&parStack, L_BRACE);
      else 
	if (c == L_BRACKET) 
	  push(&parStack, L_BRACKET);
	else 
	  if (c == R_PAREN) {
	    if (!length(parStack)) {
	      //return false;
	      printf("Parentheses not balanced RP\n");
	      return 0;
	    }
	    if (pop(&parStack) != L_PAREN) {
	      //return false;
	      printf("Parentheses not balanced LP\n");
	      return 0;
	    }
	  }
	  else 
	    if (c == R_BRACE) {
	      if (!length(parStack)) {
		//return false;
		printf("Parentheses not balanced RB\n");
		return 0;
	      }
	      if (pop(&parStack) != L_BRACE) {
		//return false;
		printf("Parentheses not balanced LB\n");
		return 0;
	      }
            }
            else 
	      if (c == R_BRACKET) {
                if (!length(parStack)){
		  //return false;
		  printf("Parentheses not balanced RBR\n");
		  return 0;
		}
                if (pop(&parStack) != L_BRACKET) {
		  //return false;
		  printf("Parentheses not balanced LBR\n");
		  return 0;
		}
	      }   
  }
  if (!length(parStack))
    printf("Parentheses are balanced\n");
  else
    printf("Parentheses not balanced\n");

  return 0;

}

